/*
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */
/*
 * This code was generated by https://code.google.com/p/google-apis-client-generator/
 * (build: 2013-06-26 16:27:34 UTC)
 * on 2013-07-31 at 17:37:43 UTC 
 * Modify at your own risk.
 */

package com.google.api.services.shopping;

/**
 * Service definition for Shopping (v1).
 *
 * <p>
 * Lets you search over product data.
 * </p>
 *
 * <p>
 * For more information about this service, see the
 * <a href="https://developers.google.com/shopping-search/v1/getting_started" target="_blank">API Documentation</a>
 * </p>
 *
 * <p>
 * This service uses {@link ShoppingRequestInitializer} to initialize global parameters via its
 * {@link Builder}.
 * </p>
 *
 * @since 1.3
 * @author Google, Inc.
 */
@SuppressWarnings("javadoc")
public class Shopping extends com.google.api.client.googleapis.services.json.AbstractGoogleJsonClient {

  // Note: Leave this static initializer at the top of the file.
  static {
    com.google.api.client.util.Preconditions.checkState(
        com.google.api.client.googleapis.GoogleUtils.MAJOR_VERSION == 1 &&
        com.google.api.client.googleapis.GoogleUtils.MINOR_VERSION >= 15,
        "You are currently running with version %s of google-api-client. " +
        "You need at least version 1.15 of google-api-client to run version " +
        "1.16.0-rc of the Search API For Shopping library.", com.google.api.client.googleapis.GoogleUtils.VERSION);
  }

  /**
   * The default encoded root URL of the service. This is determined when the library is generated
   * and normally should not be changed.
   *
   * @since 1.7
   */
  public static final String DEFAULT_ROOT_URL = "https://www.googleapis.com/";

  /**
   * The default encoded service path of the service. This is determined when the library is
   * generated and normally should not be changed.
   *
   * @since 1.7
   */
  public static final String DEFAULT_SERVICE_PATH = "shopping/search/v1/";

  /**
   * The default encoded base URL of the service. This is determined when the library is generated
   * and normally should not be changed.
   */
  public static final String DEFAULT_BASE_URL = DEFAULT_ROOT_URL + DEFAULT_SERVICE_PATH;

  /**
   * Constructor.
   *
   * <p>
   * Use {@link Builder} if you need to specify any of the optional parameters.
   * </p>
   *
   * @param transport HTTP transport, which should normally be:
   *        <ul>
   *        <li>Google App Engine:
   *        {@code com.google.api.client.extensions.appengine.http.UrlFetchTransport}</li>
   *        <li>Android: {@code newCompatibleTransport} from
   *        {@code com.google.api.client.extensions.android.http.AndroidHttp}</li>
   *        <li>Java: {@link com.google.api.client.googleapis.javanet.GoogleNetHttpTransport#newTrustedTransport()}
   *        </li>
   *        </ul>
   * @param jsonFactory JSON factory, which may be:
   *        <ul>
   *        <li>Jackson: {@code com.google.api.client.json.jackson2.JacksonFactory}</li>
   *        <li>Google GSON: {@code com.google.api.client.json.gson.GsonFactory}</li>
   *        <li>Android Honeycomb or higher:
   *        {@code com.google.api.client.extensions.android.json.AndroidJsonFactory}</li>
   *        </ul>
   * @param httpRequestInitializer HTTP request initializer or {@code null} for none
   * @since 1.7
   */
  public Shopping(com.google.api.client.http.HttpTransport transport, com.google.api.client.json.JsonFactory jsonFactory,
      com.google.api.client.http.HttpRequestInitializer httpRequestInitializer) {
    this(new Builder(transport, jsonFactory, httpRequestInitializer));
  }

  /**
   * @param builder builder
   */
  Shopping(Builder builder) {
    super(builder);
  }

  @Override
  protected void initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest<?> httpClientRequest) throws java.io.IOException {
    super.initialize(httpClientRequest);
  }

  /**
   * An accessor for creating requests from the Products collection.
   *
   * <p>The typical use is:</p>
   * <pre>
   *   {@code Shopping shopping = new Shopping(...);}
   *   {@code Shopping.Products.List request = shopping.products().list(parameters ...)}
   * </pre>
   *
   * @return the resource collection
   */
  public Products products() {
    return new Products();
  }

  /**
   * The "products" collection of methods.
   */
  public class Products {

    /**
     * Returns a single product
     *
     * Create a request for the method "products.get".
     *
     * This request holds the parameters needed by the the shopping server.  After setting any optional
     * parameters, call the {@link Get#execute()} method to invoke the remote operation.
     *
     * @param source Query source
     * @param accountId Merchant center account id
     * @param productIdType Type of productId
     * @param productId Id of product
     * @return the request
     */
    public Get get(java.lang.String source, java.lang.Long accountId, java.lang.String productIdType, java.lang.String productId) throws java.io.IOException {
      Get result = new Get(source, accountId, productIdType, productId);
      initialize(result);
      return result;
    }

    public class Get extends ShoppingRequest<com.google.api.services.shopping.model.Product> {

      private static final String REST_PATH = "{source}/products/{accountId}/{productIdType}/{productId}";

      /**
       * Returns a single product
       *
       * Create a request for the method "products.get".
       *
       * This request holds the parameters needed by the the shopping server.  After setting any
       * optional parameters, call the {@link Get#execute()} method to invoke the remote operation. <p>
       * {@link Get#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)}
       * must be called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param source Query source
       * @param accountId Merchant center account id
       * @param productIdType Type of productId
       * @param productId Id of product
       * @since 1.13
       */
      protected Get(java.lang.String source, java.lang.Long accountId, java.lang.String productIdType, java.lang.String productId) {
        super(Shopping.this, "GET", REST_PATH, null, com.google.api.services.shopping.model.Product.class);
        this.source = com.google.api.client.util.Preconditions.checkNotNull(source, "Required parameter source must be specified.");
        this.accountId = com.google.api.client.util.Preconditions.checkNotNull(accountId, "Required parameter accountId must be specified.");
        this.productIdType = com.google.api.client.util.Preconditions.checkNotNull(productIdType, "Required parameter productIdType must be specified.");
        this.productId = com.google.api.client.util.Preconditions.checkNotNull(productId, "Required parameter productId must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public Get setAlt(java.lang.String alt) {
        return (Get) super.setAlt(alt);
      }

      @Override
      public Get setFields(java.lang.String fields) {
        return (Get) super.setFields(fields);
      }

      @Override
      public Get setKey(java.lang.String key) {
        return (Get) super.setKey(key);
      }

      @Override
      public Get setOauthToken(java.lang.String oauthToken) {
        return (Get) super.setOauthToken(oauthToken);
      }

      @Override
      public Get setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (Get) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Get setQuotaUser(java.lang.String quotaUser) {
        return (Get) super.setQuotaUser(quotaUser);
      }

      @Override
      public Get setUserIp(java.lang.String userIp) {
        return (Get) super.setUserIp(userIp);
      }

      /** Query source */
      @com.google.api.client.util.Key
      private java.lang.String source;

      /** Query source
       */
      public java.lang.String getSource() {
        return source;
      }

      /** Query source */
      public Get setSource(java.lang.String source) {
        this.source = source;
        return this;
      }

      /** Merchant center account id */
      @com.google.api.client.util.Key
      private java.lang.Long accountId;

      /** Merchant center account id
       */
      public java.lang.Long getAccountId() {
        return accountId;
      }

      /** Merchant center account id */
      public Get setAccountId(java.lang.Long accountId) {
        this.accountId = accountId;
        return this;
      }

      /** Type of productId */
      @com.google.api.client.util.Key
      private java.lang.String productIdType;

      /** Type of productId
       */
      public java.lang.String getProductIdType() {
        return productIdType;
      }

      /** Type of productId */
      public Get setProductIdType(java.lang.String productIdType) {
        this.productIdType = productIdType;
        return this;
      }

      /** Id of product */
      @com.google.api.client.util.Key
      private java.lang.String productId;

      /** Id of product
       */
      public java.lang.String getProductId() {
        return productId;
      }

      /** Id of product */
      public Get setProductId(java.lang.String productId) {
        this.productId = productId;
        return this;
      }

      /** Category specification */
      @com.google.api.client.util.Key("categories.include")
      private java.lang.String categoriesInclude;

      /** Category specification
       */
      public java.lang.String getCategoriesInclude() {
        return categoriesInclude;
      }

      /** Category specification */
      public Get setCategoriesInclude(java.lang.String categoriesInclude) {
        this.categoriesInclude = categoriesInclude;
        return this;
      }

      /** Whether to return recommendation information */
      @com.google.api.client.util.Key("recommendations.enabled")
      private java.lang.Boolean recommendationsEnabled;

      /** Whether to return recommendation information
       */
      public java.lang.Boolean getRecommendationsEnabled() {
        return recommendationsEnabled;
      }

      /** Whether to return recommendation information */
      public Get setRecommendationsEnabled(java.lang.Boolean recommendationsEnabled) {
        this.recommendationsEnabled = recommendationsEnabled;
        return this;
      }

      /** Thumbnail specification */
      @com.google.api.client.util.Key
      private java.lang.String thumbnails;

      /** Thumbnail specification
       */
      public java.lang.String getThumbnails() {
        return thumbnails;
      }

      /** Thumbnail specification */
      public Get setThumbnails(java.lang.String thumbnails) {
        this.thumbnails = thumbnails;
        return this;
      }

      /** Merchant taxonomy */
      @com.google.api.client.util.Key
      private java.lang.String taxonomy;

      /** Merchant taxonomy
       */
      public java.lang.String getTaxonomy() {
        return taxonomy;
      }

      /** Merchant taxonomy */
      public Get setTaxonomy(java.lang.String taxonomy) {
        this.taxonomy = taxonomy;
        return this;
      }

      /** This parameter is currently ignored */
      @com.google.api.client.util.Key("categories.useGcsConfig")
      private java.lang.Boolean categoriesUseGcsConfig;

      /** This parameter is currently ignored
       */
      public java.lang.Boolean getCategoriesUseGcsConfig() {
        return categoriesUseGcsConfig;
      }

      /** This parameter is currently ignored */
      public Get setCategoriesUseGcsConfig(java.lang.Boolean categoriesUseGcsConfig) {
        this.categoriesUseGcsConfig = categoriesUseGcsConfig;
        return this;
      }

      /** Recommendation specification */
      @com.google.api.client.util.Key("recommendations.include")
      private java.lang.String recommendationsInclude;

      /** Recommendation specification
       */
      public java.lang.String getRecommendationsInclude() {
        return recommendationsInclude;
      }

      /** Recommendation specification */
      public Get setRecommendationsInclude(java.lang.String recommendationsInclude) {
        this.recommendationsInclude = recommendationsInclude;
        return this;
      }

      /** Whether to return category information */
      @com.google.api.client.util.Key("categories.enabled")
      private java.lang.Boolean categoriesEnabled;

      /** Whether to return category information
       */
      public java.lang.Boolean getCategoriesEnabled() {
        return categoriesEnabled;
      }

      /** Whether to return category information */
      public Get setCategoriesEnabled(java.lang.Boolean categoriesEnabled) {
        this.categoriesEnabled = categoriesEnabled;
        return this;
      }

      /** Location used to determine tax and shipping */
      @com.google.api.client.util.Key
      private java.lang.String location;

      /** Location used to determine tax and shipping
       */
      public java.lang.String getLocation() {
        return location;
      }

      /** Location used to determine tax and shipping */
      public Get setLocation(java.lang.String location) {
        this.location = location;
        return this;
      }

      /** Comma separated list of attributes to return */
      @com.google.api.client.util.Key
      private java.lang.String attributeFilter;

      /** Comma separated list of attributes to return
       */
      public java.lang.String getAttributeFilter() {
        return attributeFilter;
      }

      /** Comma separated list of attributes to return */
      public Get setAttributeFilter(java.lang.String attributeFilter) {
        this.attributeFilter = attributeFilter;
        return this;
      }

      /** This parameter is currently ignored */
      @com.google.api.client.util.Key("recommendations.useGcsConfig")
      private java.lang.Boolean recommendationsUseGcsConfig;

      /** This parameter is currently ignored
       */
      public java.lang.Boolean getRecommendationsUseGcsConfig() {
        return recommendationsUseGcsConfig;
      }

      /** This parameter is currently ignored */
      public Get setRecommendationsUseGcsConfig(java.lang.Boolean recommendationsUseGcsConfig) {
        this.recommendationsUseGcsConfig = recommendationsUseGcsConfig;
        return this;
      }

      @Override
      public Get set(String parameterName, Object value) {
        return (Get) super.set(parameterName, value);
      }
    }
    /**
     * Returns a list of products and content modules
     *
     * Create a request for the method "products.list".
     *
     * This request holds the parameters needed by the the shopping server.  After setting any optional
     * parameters, call the {@link List#execute()} method to invoke the remote operation.
     *
     * @param source Query source
     * @return the request
     */
    public List list(java.lang.String source) throws java.io.IOException {
      List result = new List(source);
      initialize(result);
      return result;
    }

    public class List extends ShoppingRequest<com.google.api.services.shopping.model.Products> {

      private static final String REST_PATH = "{source}/products";

      /**
       * Returns a list of products and content modules
       *
       * Create a request for the method "products.list".
       *
       * This request holds the parameters needed by the the shopping server.  After setting any
       * optional parameters, call the {@link List#execute()} method to invoke the remote operation. <p>
       * {@link List#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)}
       * must be called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param source Query source
       * @since 1.13
       */
      protected List(java.lang.String source) {
        super(Shopping.this, "GET", REST_PATH, null, com.google.api.services.shopping.model.Products.class);
        this.source = com.google.api.client.util.Preconditions.checkNotNull(source, "Required parameter source must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public List setAlt(java.lang.String alt) {
        return (List) super.setAlt(alt);
      }

      @Override
      public List setFields(java.lang.String fields) {
        return (List) super.setFields(fields);
      }

      @Override
      public List setKey(java.lang.String key) {
        return (List) super.setKey(key);
      }

      @Override
      public List setOauthToken(java.lang.String oauthToken) {
        return (List) super.setOauthToken(oauthToken);
      }

      @Override
      public List setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (List) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public List setQuotaUser(java.lang.String quotaUser) {
        return (List) super.setQuotaUser(quotaUser);
      }

      @Override
      public List setUserIp(java.lang.String userIp) {
        return (List) super.setUserIp(userIp);
      }

      /** Query source */
      @com.google.api.client.util.Key
      private java.lang.String source;

      /** Query source
       */
      public java.lang.String getSource() {
        return source;
      }

      /** Query source */
      public List setSource(java.lang.String source) {
        this.source = source;
        return this;
      }

      /** Facets to include (applies when useGcsConfig == false) */
      @com.google.api.client.util.Key("facets.include")
      private java.lang.String facetsInclude;

      /** Facets to include (applies when useGcsConfig == false)
       */
      public java.lang.String getFacetsInclude() {
        return facetsInclude;
      }

      /** Facets to include (applies when useGcsConfig == false) */
      public List setFacetsInclude(java.lang.String facetsInclude) {
        this.facetsInclude = facetsInclude;
        return this;
      }

      /** Category for which to retrieve recommendations */
      @com.google.api.client.util.Key("categoryRecommendations.category")
      private java.lang.String categoryRecommendationsCategory;

      /** Category for which to retrieve recommendations
       */
      public java.lang.String getCategoryRecommendationsCategory() {
        return categoryRecommendationsCategory;
      }

      /** Category for which to retrieve recommendations */
      public List setCategoryRecommendationsCategory(java.lang.String categoryRecommendationsCategory) {
        this.categoryRecommendationsCategory = categoryRecommendationsCategory;
        return this;
      }

      /** Whether to return extra information. */
      @com.google.api.client.util.Key("extras.enabled")
      private java.lang.Boolean extrasEnabled;

      /** Whether to return extra information.
       */
      public java.lang.Boolean getExtrasEnabled() {
        return extrasEnabled;
      }

      /** Whether to return extra information. */
      public List setExtrasEnabled(java.lang.Boolean extrasEnabled) {
        this.extrasEnabled = extrasEnabled;
        return this;
      }

      /** Whether to return facet information */
      @com.google.api.client.util.Key("facets.enabled")
      private java.lang.Boolean facetsEnabled;

      /** Whether to return facet information
       */
      public java.lang.Boolean getFacetsEnabled() {
        return facetsEnabled;
      }

      /** Whether to return facet information */
      public List setFacetsEnabled(java.lang.Boolean facetsEnabled) {
        this.facetsEnabled = facetsEnabled;
        return this;
      }

      /** This parameter is currently ignored */
      @com.google.api.client.util.Key("relatedQueries.useGcsConfig")
      private java.lang.Boolean relatedQueriesUseGcsConfig;

      /** This parameter is currently ignored
       */
      public java.lang.Boolean getRelatedQueriesUseGcsConfig() {
        return relatedQueriesUseGcsConfig;
      }

      /** This parameter is currently ignored */
      public List setRelatedQueriesUseGcsConfig(java.lang.Boolean relatedQueriesUseGcsConfig) {
        this.relatedQueriesUseGcsConfig = relatedQueriesUseGcsConfig;
        return this;
      }

      /** Whether to return promotion information */
      @com.google.api.client.util.Key("promotions.enabled")
      private java.lang.Boolean promotionsEnabled;

      /** Whether to return promotion information
       */
      public java.lang.Boolean getPromotionsEnabled() {
        return promotionsEnabled;
      }

      /** Whether to return promotion information */
      public List setPromotionsEnabled(java.lang.Boolean promotionsEnabled) {
        this.promotionsEnabled = promotionsEnabled;
        return this;
      }

      /** Channels specification */
      @com.google.api.client.util.Key
      private java.lang.String channels;

      /** Channels specification
       */
      public java.lang.String getChannels() {
        return channels;
      }

      /** Channels specification */
      public List setChannels(java.lang.String channels) {
        this.channels = channels;
        return this;
      }

      /** Currency restriction (ISO 4217) */
      @com.google.api.client.util.Key
      private java.lang.String currency;

      /** Currency restriction (ISO 4217)
       */
      public java.lang.String getCurrency() {
        return currency;
      }

      /** Currency restriction (ISO 4217) */
      public List setCurrency(java.lang.String currency) {
        this.currency = currency;
        return this;
      }

      /** Whether to return category recommendation information */
      @com.google.api.client.util.Key("categoryRecommendations.enabled")
      private java.lang.Boolean categoryRecommendationsEnabled;

      /** Whether to return category recommendation information
       */
      public java.lang.Boolean getCategoryRecommendationsEnabled() {
        return categoryRecommendationsEnabled;
      }

      /** Whether to return category recommendation information */
      public List setCategoryRecommendationsEnabled(java.lang.Boolean categoryRecommendationsEnabled) {
        this.categoryRecommendationsEnabled = categoryRecommendationsEnabled;
        return this;
      }

      /** Facets to discover */
      @com.google.api.client.util.Key("facets.discover")
      private java.lang.String facetsDiscover;

      /** Facets to discover
       */
      public java.lang.String getFacetsDiscover() {
        return facetsDiscover;
      }

      /** Facets to discover */
      public List setFacetsDiscover(java.lang.String facetsDiscover) {
        this.facetsDiscover = facetsDiscover;
        return this;
      }

      /** What extra information to return. */
      @com.google.api.client.util.Key("extras.info")
      private java.lang.String extrasInfo;

      /** What extra information to return.
       */
      public java.lang.String getExtrasInfo() {
        return extrasInfo;
      }

      /** What extra information to return. */
      public List setExtrasInfo(java.lang.String extrasInfo) {
        this.extrasInfo = extrasInfo;
        return this;
      }

      /** Index (1-based) of first product to return */
      @com.google.api.client.util.Key
      private java.lang.Long startIndex;

      /** Index (1-based) of first product to return
       */
      public java.lang.Long getStartIndex() {
        return startIndex;
      }

      /** Index (1-based) of first product to return */
      public List setStartIndex(java.lang.Long startIndex) {
        this.startIndex = startIndex;
        return this;
      }

      /**
       * Comma separated list of availabilities (outOfStock, limited, inStock, backOrder, preOrder,
       * onDisplayToOrder) to return
       */
      @com.google.api.client.util.Key
      private java.lang.String availability;

      /** Comma separated list of availabilities (outOfStock, limited, inStock, backOrder, preOrder,
     onDisplayToOrder) to return
       */
      public java.lang.String getAvailability() {
        return availability;
      }

      /**
       * Comma separated list of availabilities (outOfStock, limited, inStock, backOrder, preOrder,
       * onDisplayToOrder) to return
       */
      public List setAvailability(java.lang.String availability) {
        this.availability = availability;
        return this;
      }

      /** Crowding specification */
      @com.google.api.client.util.Key
      private java.lang.String crowdBy;

      /** Crowding specification
       */
      public java.lang.String getCrowdBy() {
        return crowdBy;
      }

      /** Crowding specification */
      public List setCrowdBy(java.lang.String crowdBy) {
        this.crowdBy = crowdBy;
        return this;
      }

      /** Search query */
      @com.google.api.client.util.Key
      private java.lang.String q;

      /** Search query
       */
      public java.lang.String getQ() {
        return q;
      }

      /** Search query */
      public List setQ(java.lang.String q) {
        this.q = q;
        return this;
      }

      /** Whether to return spelling suggestions */
      @com.google.api.client.util.Key("spelling.enabled")
      private java.lang.Boolean spellingEnabled;

      /** Whether to return spelling suggestions
       */
      public java.lang.Boolean getSpellingEnabled() {
        return spellingEnabled;
      }

      /** Whether to return spelling suggestions */
      public List setSpellingEnabled(java.lang.Boolean spellingEnabled) {
        this.spellingEnabled = spellingEnabled;
        return this;
      }

      /** Taxonomy name */
      @com.google.api.client.util.Key
      private java.lang.String taxonomy;

      /** Taxonomy name
       */
      public java.lang.String getTaxonomy() {
        return taxonomy;
      }

      /** Taxonomy name */
      public List setTaxonomy(java.lang.String taxonomy) {
        this.taxonomy = taxonomy;
        return this;
      }

      /** This parameter is currently ignored */
      @com.google.api.client.util.Key("spelling.useGcsConfig")
      private java.lang.Boolean spellingUseGcsConfig;

      /** This parameter is currently ignored
       */
      public java.lang.Boolean getSpellingUseGcsConfig() {
        return spellingUseGcsConfig;
      }

      /** This parameter is currently ignored */
      public List setSpellingUseGcsConfig(java.lang.Boolean spellingUseGcsConfig) {
        this.spellingUseGcsConfig = spellingUseGcsConfig;
        return this;
      }

      /** One of CommerceSearchUseCase, ShoppingApiUseCase */
      @com.google.api.client.util.Key
      private java.lang.String useCase;

      /** One of CommerceSearchUseCase, ShoppingApiUseCase
       */
      public java.lang.String getUseCase() {
        return useCase;
      }

      /** One of CommerceSearchUseCase, ShoppingApiUseCase */
      public List setUseCase(java.lang.String useCase) {
        this.useCase = useCase;
        return this;
      }

      /** Location used to determine tax and shipping */
      @com.google.api.client.util.Key
      private java.lang.String location;

      /** Location used to determine tax and shipping
       */
      public java.lang.String getLocation() {
        return location;
      }

      /** Location used to determine tax and shipping */
      public List setLocation(java.lang.String location) {
        this.location = location;
        return this;
      }

      /** Maximum number of variant results to return per result */
      @com.google.api.client.util.Key
      private java.lang.Integer maxVariants;

      /** Maximum number of variant results to return per result
       */
      public java.lang.Integer getMaxVariants() {
        return maxVariants;
      }

      /** Maximum number of variant results to return per result */
      public List setMaxVariants(java.lang.Integer maxVariants) {
        this.maxVariants = maxVariants;
        return this;
      }

      /** Category specification */
      @com.google.api.client.util.Key("categories.include")
      private java.lang.String categoriesInclude;

      /** Category specification
       */
      public java.lang.String getCategoriesInclude() {
        return categoriesInclude;
      }

      /** Category specification */
      public List setCategoriesInclude(java.lang.String categoriesInclude) {
        this.categoriesInclude = categoriesInclude;
        return this;
      }

      /** Boosting specification */
      @com.google.api.client.util.Key
      private java.lang.String boostBy;

      /** Boosting specification
       */
      public java.lang.String getBoostBy() {
        return boostBy;
      }

      /** Boosting specification */
      public List setBoostBy(java.lang.String boostBy) {
        this.boostBy = boostBy;
        return this;
      }

      /** Whether safe search is enabled. Default: true */
      @com.google.api.client.util.Key
      private java.lang.Boolean safe;

      /** Whether safe search is enabled. Default: true
       */
      public java.lang.Boolean getSafe() {
        return safe;
      }

      /** Whether safe search is enabled. Default: true */
      public List setSafe(java.lang.Boolean safe) {
        this.safe = safe;
        return this;
      }

      /** This parameter is currently ignored */
      @com.google.api.client.util.Key("categories.useGcsConfig")
      private java.lang.Boolean categoriesUseGcsConfig;

      /** This parameter is currently ignored
       */
      public java.lang.Boolean getCategoriesUseGcsConfig() {
        return categoriesUseGcsConfig;
      }

      /** This parameter is currently ignored */
      public List setCategoriesUseGcsConfig(java.lang.Boolean categoriesUseGcsConfig) {
        this.categoriesUseGcsConfig = categoriesUseGcsConfig;
        return this;
      }

      /** Maximum number of results to return */
      @com.google.api.client.util.Key
      private java.lang.Long maxResults;

      /** Maximum number of results to return
       */
      public java.lang.Long getMaxResults() {
        return maxResults;
      }

      /** Maximum number of results to return */
      public List setMaxResults(java.lang.Long maxResults) {
        this.maxResults = maxResults;
        return this;
      }

      /** Whether to return facet information as configured in the GCS account */
      @com.google.api.client.util.Key("facets.useGcsConfig")
      private java.lang.Boolean facetsUseGcsConfig;

      /** Whether to return facet information as configured in the GCS account
       */
      public java.lang.Boolean getFacetsUseGcsConfig() {
        return facetsUseGcsConfig;
      }

      /** Whether to return facet information as configured in the GCS account */
      public List setFacetsUseGcsConfig(java.lang.Boolean facetsUseGcsConfig) {
        this.facetsUseGcsConfig = facetsUseGcsConfig;
        return this;
      }

      /** Whether to return category information */
      @com.google.api.client.util.Key("categories.enabled")
      private java.lang.Boolean categoriesEnabled;

      /** Whether to return category information
       */
      public java.lang.Boolean getCategoriesEnabled() {
        return categoriesEnabled;
      }

      /** Whether to return category information */
      public List setCategoriesEnabled(java.lang.Boolean categoriesEnabled) {
        this.categoriesEnabled = categoriesEnabled;
        return this;
      }

      /** Comma separated list of attributes to return */
      @com.google.api.client.util.Key
      private java.lang.String attributeFilter;

      /** Comma separated list of attributes to return
       */
      public java.lang.String getAttributeFilter() {
        return attributeFilter;
      }

      /** Comma separated list of attributes to return */
      public List setAttributeFilter(java.lang.String attributeFilter) {
        this.attributeFilter = attributeFilter;
        return this;
      }

      /** Whether to add a click tracking parameter to offer URLs */
      @com.google.api.client.util.Key
      private java.lang.Boolean clickTracking;

      /** Whether to add a click tracking parameter to offer URLs
       */
      public java.lang.Boolean getClickTracking() {
        return clickTracking;
      }

      /** Whether to add a click tracking parameter to offer URLs */
      public List setClickTracking(java.lang.Boolean clickTracking) {
        this.clickTracking = clickTracking;
        return this;
      }

      /** Image thumbnails specification */
      @com.google.api.client.util.Key
      private java.lang.String thumbnails;

      /** Image thumbnails specification
       */
      public java.lang.String getThumbnails() {
        return thumbnails;
      }

      /** Image thumbnails specification */
      public List setThumbnails(java.lang.String thumbnails) {
        this.thumbnails = thumbnails;
        return this;
      }

      /** Language restriction (BCP 47) */
      @com.google.api.client.util.Key
      private java.lang.String language;

      /** Language restriction (BCP 47)
       */
      public java.lang.String getLanguage() {
        return language;
      }

      /** Language restriction (BCP 47) */
      public List setLanguage(java.lang.String language) {
        this.language = language;
        return this;
      }

      /** Category recommendation specification */
      @com.google.api.client.util.Key("categoryRecommendations.include")
      private java.lang.String categoryRecommendationsInclude;

      /** Category recommendation specification
       */
      public java.lang.String getCategoryRecommendationsInclude() {
        return categoryRecommendationsInclude;
      }

      /** Category recommendation specification */
      public List setCategoryRecommendationsInclude(java.lang.String categoryRecommendationsInclude) {
        this.categoryRecommendationsInclude = categoryRecommendationsInclude;
        return this;
      }

      /** Country restriction (ISO 3166) */
      @com.google.api.client.util.Key
      private java.lang.String country;

      /** Country restriction (ISO 3166)
       */
      public java.lang.String getCountry() {
        return country;
      }

      /** Country restriction (ISO 3166) */
      public List setCountry(java.lang.String country) {
        this.country = country;
        return this;
      }

      /** Ranking specification */
      @com.google.api.client.util.Key
      private java.lang.String rankBy;

      /** Ranking specification
       */
      public java.lang.String getRankBy() {
        return rankBy;
      }

      /** Ranking specification */
      public List setRankBy(java.lang.String rankBy) {
        this.rankBy = rankBy;
        return this;
      }

      /** Restriction specification */
      @com.google.api.client.util.Key
      private java.lang.String restrictBy;

      /** Restriction specification
       */
      public java.lang.String getRestrictBy() {
        return restrictBy;
      }

      /** Restriction specification */
      public List setRestrictBy(java.lang.String restrictBy) {
        this.restrictBy = restrictBy;
        return this;
      }

      /** Return empty facet buckets. */
      @com.google.api.client.util.Key("facets.includeEmptyBuckets")
      private java.lang.Boolean facetsIncludeEmptyBuckets;

      /** Return empty facet buckets.
       */
      public java.lang.Boolean getFacetsIncludeEmptyBuckets() {
        return facetsIncludeEmptyBuckets;
      }

      /** Return empty facet buckets. */
      public List setFacetsIncludeEmptyBuckets(java.lang.Boolean facetsIncludeEmptyBuckets) {
        this.facetsIncludeEmptyBuckets = facetsIncludeEmptyBuckets;
        return this;
      }

      /** Whether to return redirect information */
      @com.google.api.client.util.Key("redirects.enabled")
      private java.lang.Boolean redirectsEnabled;

      /** Whether to return redirect information
       */
      public java.lang.Boolean getRedirectsEnabled() {
        return redirectsEnabled;
      }

      /** Whether to return redirect information */
      public List setRedirectsEnabled(java.lang.Boolean redirectsEnabled) {
        this.redirectsEnabled = redirectsEnabled;
        return this;
      }

      /** Whether to return redirect information as configured in the GCS account */
      @com.google.api.client.util.Key("redirects.useGcsConfig")
      private java.lang.Boolean redirectsUseGcsConfig;

      /** Whether to return redirect information as configured in the GCS account
       */
      public java.lang.Boolean getRedirectsUseGcsConfig() {
        return redirectsUseGcsConfig;
      }

      /** Whether to return redirect information as configured in the GCS account */
      public List setRedirectsUseGcsConfig(java.lang.Boolean redirectsUseGcsConfig) {
        this.redirectsUseGcsConfig = redirectsUseGcsConfig;
        return this;
      }

      /** Whether to return related queries */
      @com.google.api.client.util.Key("relatedQueries.enabled")
      private java.lang.Boolean relatedQueriesEnabled;

      /** Whether to return related queries
       */
      public java.lang.Boolean getRelatedQueriesEnabled() {
        return relatedQueriesEnabled;
      }

      /** Whether to return related queries */
      public List setRelatedQueriesEnabled(java.lang.Boolean relatedQueriesEnabled) {
        this.relatedQueriesEnabled = relatedQueriesEnabled;
        return this;
      }

      /** This parameter is currently ignored */
      @com.google.api.client.util.Key("categoryRecommendations.useGcsConfig")
      private java.lang.Boolean categoryRecommendationsUseGcsConfig;

      /** This parameter is currently ignored
       */
      public java.lang.Boolean getCategoryRecommendationsUseGcsConfig() {
        return categoryRecommendationsUseGcsConfig;
      }

      /** This parameter is currently ignored */
      public List setCategoryRecommendationsUseGcsConfig(java.lang.Boolean categoryRecommendationsUseGcsConfig) {
        this.categoryRecommendationsUseGcsConfig = categoryRecommendationsUseGcsConfig;
        return this;
      }

      /** Whether to return promotion information as configured in the GCS account */
      @com.google.api.client.util.Key("promotions.useGcsConfig")
      private java.lang.Boolean promotionsUseGcsConfig;

      /** Whether to return promotion information as configured in the GCS account
       */
      public java.lang.Boolean getPromotionsUseGcsConfig() {
        return promotionsUseGcsConfig;
      }

      /** Whether to return promotion information as configured in the GCS account */
      public List setPromotionsUseGcsConfig(java.lang.Boolean promotionsUseGcsConfig) {
        this.promotionsUseGcsConfig = promotionsUseGcsConfig;
        return this;
      }

      @Override
      public List set(String parameterName, Object value) {
        return (List) super.set(parameterName, value);
      }
    }

  }

  /**
   * Builder for {@link Shopping}.
   *
   * <p>
   * Implementation is not thread-safe.
   * </p>
   *
   * @since 1.3.0
   */
  public static final class Builder extends com.google.api.client.googleapis.services.json.AbstractGoogleJsonClient.Builder {

    /**
     * Returns an instance of a new builder.
     *
     * @param transport HTTP transport, which should normally be:
     *        <ul>
     *        <li>Google App Engine:
     *        {@code com.google.api.client.extensions.appengine.http.UrlFetchTransport}</li>
     *        <li>Android: {@code newCompatibleTransport} from
     *        {@code com.google.api.client.extensions.android.http.AndroidHttp}</li>
     *        <li>Java: {@link com.google.api.client.googleapis.javanet.GoogleNetHttpTransport#newTrustedTransport()}
     *        </li>
     *        </ul>
     * @param jsonFactory JSON factory, which may be:
     *        <ul>
     *        <li>Jackson: {@code com.google.api.client.json.jackson2.JacksonFactory}</li>
     *        <li>Google GSON: {@code com.google.api.client.json.gson.GsonFactory}</li>
     *        <li>Android Honeycomb or higher:
     *        {@code com.google.api.client.extensions.android.json.AndroidJsonFactory}</li>
     *        </ul>
     * @param httpRequestInitializer HTTP request initializer or {@code null} for none
     * @since 1.7
     */
    public Builder(com.google.api.client.http.HttpTransport transport, com.google.api.client.json.JsonFactory jsonFactory,
        com.google.api.client.http.HttpRequestInitializer httpRequestInitializer) {
      super(
          transport,
          jsonFactory,
          DEFAULT_ROOT_URL,
          DEFAULT_SERVICE_PATH,
          httpRequestInitializer,
          false);
    }

    /** Builds a new instance of {@link Shopping}. */
    @Override
    public Shopping build() {
      return new Shopping(this);
    }

    @Override
    public Builder setRootUrl(String rootUrl) {
      return (Builder) super.setRootUrl(rootUrl);
    }

    @Override
    public Builder setServicePath(String servicePath) {
      return (Builder) super.setServicePath(servicePath);
    }

    @Override
    public Builder setHttpRequestInitializer(com.google.api.client.http.HttpRequestInitializer httpRequestInitializer) {
      return (Builder) super.setHttpRequestInitializer(httpRequestInitializer);
    }

    @Override
    public Builder setApplicationName(String applicationName) {
      return (Builder) super.setApplicationName(applicationName);
    }

    @Override
    public Builder setSuppressPatternChecks(boolean suppressPatternChecks) {
      return (Builder) super.setSuppressPatternChecks(suppressPatternChecks);
    }

    @Override
    public Builder setSuppressRequiredParameterChecks(boolean suppressRequiredParameterChecks) {
      return (Builder) super.setSuppressRequiredParameterChecks(suppressRequiredParameterChecks);
    }

    @Override
    public Builder setSuppressAllChecks(boolean suppressAllChecks) {
      return (Builder) super.setSuppressAllChecks(suppressAllChecks);
    }

    /**
     * Set the {@link ShoppingRequestInitializer}.
     *
     * @since 1.12
     */
    public Builder setShoppingRequestInitializer(
        ShoppingRequestInitializer shoppingRequestInitializer) {
      return (Builder) super.setGoogleClientRequestInitializer(shoppingRequestInitializer);
    }

    @Override
    public Builder setGoogleClientRequestInitializer(
        com.google.api.client.googleapis.services.GoogleClientRequestInitializer googleClientRequestInitializer) {
      return (Builder) super.setGoogleClientRequestInitializer(googleClientRequestInitializer);
    }
  }
}
